import xarray as xr
import pandas as pd
import pathlib
import sys
import argparse


from datetime import datetime


GLOBAL_ATTRS = {"AEROCOM":{
    'title': 'Model output for AeroCom',
    'source': 'TM5-mp',
    'institution': 'Royal Netherlands Meteorological Institute (KNMI), De Bilt, The Netherlands)',
    'contact': 'Twan van Noije; noije@knmi.nl',
    'project_id': 'AeroCom Phase 3',
    'conventions': 'CF-1.0 - HTAP',
    'time_units': 'days since 2001-01-01 00:00',
    'references': 'http://tm5.sourceforge.net/',
    'history': 'Postprocessing performed at BSC with xarray to combine time variables and output as single variable'}
 }


def split_ds_on_variables_keeping_variable_attrs(inpath, outdir, model_name, sim_name, year, datavar="AEROCOM"):
    """
    A function that splits a netcdf with multiple variables and 
    returns multiple netcdfs each containing a single variable, 
    without altering the attributes.
    """
    
    # Open the dataset
    ds = xr.open_mfdataset(inpath)
    
    
    # Save the global attributes to be appended later
    global_attrs = GLOBAL_ATTRS[datavar]
    # Run through the list of dataset variables
    for var in list(ds.keys()):
        # Create a datarray containing only one variable, keeping the coordinates and dimensions
        # Convert to a dataset
        ds_var = ds[var].to_dataset()
        
        if len(ds_var.dims) == 4:
            var_dims = "ModelLevel"
        elif len(ds_var.dims) == 3:
            var_dims = "Surface"
        if datavar == "AEROCOM":
            base_outname = "aerocom3_{}_DURF-{}_{}_{}_{}_monthly.nc".format(model_name, sim_name, var, var_dims, year)
        
        
        # Append global attrs
        ds_var.attrs = global_attrs
        outname = outdir.joinpath(base_outname)
        # Output desired single variable dataset to outdirectory
        print("Ouputting file {}".format(outname))
        ds_var.to_netcdf(outname)


def getOptions(args=sys.argv[1:]):
    parser = argparse.ArgumentParser(description="Parses command.")
    parser.add_argument("-e", "--enddate", type=str, help="Enddate")
    parser.add_argument("-s", "--startdate", type=str, help="Startdate")
    parser.add_argument("-i", "--indir", type=str, help="In directory - e.g. original files.")
    parser.add_argument("-o", "--outdir", type=str, help="Out directory hourly files.") 
    parser.add_argument("-v", "--datavar", type=str, help="Project type, e.g. AEROCOM")
    parser.add_argument("-x", "--expid", type=str, help="Expid if appropriate")
    parser.add_argument("-m", "--model_name", type=str, help="Model name")
    parser.add_argument("-sn", "--sim_name", type=str, help="Sim name")

    options = parser.parse_args(args)
    return options
        
        
if __name__ == "__main__":

    options = getOptions()
    
    
    expid = options.expid
    model_name = options.model_name
    sim_name = options.sim_name
    datavar = options.datavar
    startdate = options.startdate
    enddate = options.enddate
    indir = pathlib.Path(options.indir)
    outdir = pathlib.Path(options.outdir)
    
    
    for year_end in pd.date_range(startdate, enddate, freq="Y"):
        year = year_end.strftime("%Y")
        print(year)
        # I need to split up the data per project
        if datavar == "DOMOS":
            inpath = "*_daily_{}*".format(year)
        elif datavar == "AEROCOM":
            inpath = indir.glob("aerocom3_TM5_{}_global_{}*".format(expid, year))
    
    
    
    split_ds_on_variables_keeping_variable_attrs(inpath, outdir, model_name, sim_name, year, datavar="AEROCOM")
    

    